import mock
import unittest

from werkzeug.routing import Map, Rule
from werkzeug.test import Client
from werkzeug.wrappers import Response, BaseResponse

from lymph.testing import WebServiceTestCase
from lymph.web.interfaces import WebServiceInterface
from lymph.web.handlers import RequestHandler
from lymph.web.routing import HandledRule


class RuleHandler(RequestHandler):
    def get(self):
        return Response("Rule Handler")


class HandledRuleHandler(RequestHandler):
    def get(self):
        return Response("Handled Rule Handler")


class Web(WebServiceInterface):
    url_map = Map([
        Rule("/test/", endpoint="test"),
        Rule("/foo/", endpoint=RuleHandler),
        HandledRule("/bar/", endpoint="bar", handler=HandledRuleHandler),
        Rule("/fail/", endpoint="fail"),
        Rule("/fail-wrong-endpoint/", endpoint=42),
    ])

    def test(self, request):
        return Response("method test")


class WebIntegrationTest(WebServiceTestCase):

    service_class = Web

    def test_dispatch_rule_with_string_endpoint(self):
        response = self.client.get("/test/")
        self.assertEqual(response.data.decode("utf8"), "method test")
        self.assertEqual(response.status_code, 200)

    def test_dispatch_rule_with_callable_endpoint(self):
        response = self.client.get("/foo/")
        self.assertEqual(response.data.decode("utf8"), "Rule Handler")
        self.assertEqual(response.status_code, 200)

    def test_dispatch_handled_rule(self):
        response = self.client.get("/bar/")
        self.assertEqual(response.data.decode("utf8"), "Handled Rule Handler")
        self.assertEqual(response.status_code, 200)

    def test_dispatch_failing_rule_to_500(self):
        response = self.client.get("/fail/")
        self.assertEqual(response.data.decode("utf8"), "")
        self.assertEqual(response.status_code, 500)

    def test_dispatch_failing_endpoint_to_500(self):
        response = self.client.get("/fail-wrong-endpoint/")
        self.assertEqual(response.data.decode("utf8"), "")
        self.assertEqual(response.status_code, 500)



import gevent

from kazoo.client import KazooClient
from kazoo.handlers.gevent import SequentialGeventHandler

from lymph.core.decorators import rpc
from lymph.core.interfaces import Interface
from lymph.discovery.zookeeper import ZookeeperServiceRegistry
from lymph.events.null import NullEventSystem
from lymph.testing import LymphIntegrationTestCase


class Upper(Interface):
    service_type = 'upper'

    @rpc()
    def upper(self, text=None):
        return text.upper()


class ZookeeperIntegrationTest(LymphIntegrationTestCase):
    use_zookeeper = True

    def setUp(self):
        super(ZookeeperIntegrationTest, self).setUp()
        self.events = NullEventSystem()

        self.upper_container, interface = self.create_container(Upper, 'upper')
        self.lymph_client = self.create_client()

    def create_registry(self, **kwargs):
        zkclient = KazooClient(self.hosts, handler=SequentialGeventHandler())
        return ZookeeperServiceRegistry(zkclient)

    def test_lookup(self):
        service = self.lymph_client.container.lookup('upper')
        self.assertEqual(len(service), 1)
        self.assertEqual(list(service)[0].endpoint, self.upper_container.endpoint)

    def test_upper(self):
        reply = self.lymph_client.request('upper', 'upper.upper', {'text': 'foo'})
        self.assertEqual(reply.body, 'FOO')

    def test_ping(self):
        reply = self.lymph_client.request('upper', 'lymph.ping', {'payload': 42})
        self.assertEqual(reply.body, 42)

    def test_status(self):
        reply = self.lymph_client.request('upper', 'lymph.status', {})
        self.assertEqual(reply.body, {
            'endpoint': self.upper_container.endpoint,
            'identity': self.upper_container.identity,
        })

    def test_get_metrics(self):
        reply = self.lymph_client.request('upper', 'lymph.get_metrics', {})
        self.assertIsInstance(reply.body, list)

    def test_connection_loss(self):
        service = self.lymph_client.container.lookup('upper')
        self.assertEqual(
            [i.identity for i in service],
            [self.upper_container.identity],
        )
        self.upper_container.service_registry.client.stop()
        self.upper_container.service_registry.client.start()
        gevent.sleep(.1)  # XXX: give zk a chance to reconnect
        self.assertEqual(
            [i.identity for i in service],
            [self.upper_container.identity],
        )



from .base import BaseServiceRegistry
from lymph.exceptions import LookupFailure


class StaticServiceRegistryHub(object):
    def __init__(self):
        self.registry = {}

    def create_registry(self):
        return StaticServiceRegistry(self)

    def lookup(self, service, **kwargs):
        service_name = service.name
        try:
            instances = self.registry[service_name]
            for instance in instances:
                service.update(instance.identity, endpoint=instance.endpoint)
        except KeyError:
            raise LookupFailure()
        return service

    def register(self, service_name, instance):
        self.registry.setdefault(service_name, []).append(instance)

    def unregister(self, service_name, instance):
        self.registry.get(service_name, []).remove(instance)

    def discover(self):
        return list(self.registry.keys())


class StaticServiceRegistry(BaseServiceRegistry):
    def __init__(self, hub=None):
        super(StaticServiceRegistry, self).__init__()
        self.hub = hub or StaticServiceRegistryHub()

    def discover(self):
        return self.hub.discover()

    def lookup(self, service, **kwargs):
        return self.hub.lookup(service, **kwargs)

    def register(self, service_name, instance):
        return self.hub.register(service_name, instance)

    def unregister(self, service_name, instance):
        return self.hub.unregister(service_name, instance)



import collections

import abc
import copy
import re
import six
import yaml

from lymph.utils import import_object, Undefined
from lymph.exceptions import ConfigurationError


@six.add_metaclass(abc.ABCMeta)
class ConfigObject(collections.Mapping):

    @abc.abstractmethod
    def get(self, key, default=None):
        raise NotImplementedError()

    @abc.abstractmethod
    def get_raw(self, key, default=None):
        raise NotImplementedError()

    @abc.abstractmethod
    def set(self, key, value):
        raise NotImplementedError()

    def __getitem__(self, key):
        return self.get(key)

    def __contains__(self, key):
        try:
            self.get_raw(key)
        except KeyError:
            return False
        return True

    def setdefault(self, key, default):
        value = self.get(key)
        if value is None:
            self.set(key, default)
            return default
        return value

    def create_instance(self, key, default_class=None, **kwargs):
        instance_config = self.get(key, {})
        return self._create_instance(key, instance_config, default_class=default_class, **kwargs)

    def _create_instance(self, key, instance_config, default_class=None, **kwargs):
        if instance_config is None:
            raise ConfigurationError("no config available for %r" % key)

        clspath = instance_config.get('class', default_class)

        if clspath is None:
            raise ConfigurationError("no config available for %r (or no class configured)" % key)

        if callable(clspath):
            cls = clspath
        else:
            cls = import_object(clspath)

        if hasattr(cls, 'from_config'):
            return cls.from_config(instance_config, **kwargs)
        else:
            instance_config = copy.deepcopy(dict(instance_config))
            instance_config.pop('class', None)
            instance_config.update(kwargs)
            return cls(**instance_config)

    def get_instance(self, key, default_class=None, **kwargs):
        instance_data = self.get(key)
        if isinstance(instance_data, six.string_types) and instance_data.startswith('dep:'):
            _, dep_name = instance_data.split(':', 1)
            return self.get_dependency(dep_name, **kwargs)

        instance = self.root._instances_cache.get(key)
        if not instance:
            instance = self._create_instance(
                key, instance_data, default_class=default_class, **kwargs)
            self.root._instances_cache[key] = instance
        return instance

    def get_dependency(self, key, **kwargs):
        return self.root.get_instance('dependencies.%s' % key, **kwargs)


class ConfigView(ConfigObject):
    def __init__(self, config, path):
        self.root = config
        self.path = path

    def __len__(self):
        return len(self.root.get_raw(self.path))

    def get_raw(self, key, default=Undefined):
        return self.root.get_raw('%s.%s' % (self.path, key), default)

    def get(self, key, default=None):
        return self.root.get('%s.%s' % (self.path, key), default)

    def set(self, key, value):
        return self.root.set('%s.%s' % (self.path, key), value)

    def __iter__(self):
        return iter(self.root.get_raw(self.path))

    def __str__(self):
        return '%s -> %r' % (self.path, self.root)

    def __repr__(self):
        return '%s(%r, %r)' % (self.__class__.__name__, self.root, self.path)


_dollar_var_re = re.compile(r'\$\((\w+)\.([\w.]+)\)')


def _replace_dollar_vars(obj, namespaces):
    if isinstance(obj, dict):
        return {key: _replace_dollar_vars(value, namespaces) for key, value in obj.items()}
    elif isinstance(obj, (tuple, list)):
        return [_replace_dollar_vars(value, namespaces) for value in obj]
    elif not isinstance(obj, (six.text_type, six.string_types)):
        return obj

    match = _dollar_var_re.match(obj)
    if match and match.group(0) == obj:
        try:
            return namespaces[match.group(1)][match.group(2)]
        except KeyError:
            raise ConfigurationError('Undefined environment variable %s' % obj)

    def replace(match):
        try:
            return type(obj)(namespaces[match.group(1)][match.group(2)])
        except KeyError:
            return ConfigurationError('Undefined environment variable %s' % match.group(0))
    return _dollar_var_re.sub(replace, obj)


class Configuration(ConfigObject):
    def __init__(self, values=None, **env_vars):
        self.values = _replace_dollar_vars(values or {}, env_vars)
        self.env_vars = env_vars
        self._instances_cache = {}

    def __iter__(self):
        return iter(self.values)

    def __len__(self):
        return len(self.values)

    @property
    def root(self):
        return self

    def load_file(self, filename, sections=None):
        with open(filename, 'r') as f:
            self.load(f, sections=sections)

    def load(self, f, sections=None):
        values = _replace_dollar_vars(yaml.load(f), self.env_vars)
        for section, values in six.iteritems(values):
            if sections is None or section in sections:
                if section in self.values:
                    self.values[section].update(values)
                else:
                    self.values[section] = values

    def update(self, data):
        self.values.update(data)

    def set(self, key, data):
        path = key.split('.')
        values = self.values
        for bit in path[:-1]:
            new_values = values.setdefault(bit, {})
            if new_values is None:
                values[bit] = {}
                values = values[bit]
            else:
                values = new_values
        values[path[-1]] = data

    def get_raw(self, key, default=Undefined):
        path = key.split('.')
        values = self.values
        for bit in path[:-1]:
            values = values[bit]
            if values is None:
                if default is not Undefined:
                    return default
                raise KeyError(key)
        try:
            return values[path[-1]]
        except KeyError:
            if default is not Undefined:
                return default
            raise KeyError(key)

    def get(self, key, default=None):
        try:
            value = self.get_raw(key)
        except KeyError:
            return default
        if isinstance(value, dict):
            value = ConfigView(self, key)
        return value

    def __str__(self):
        return str(self.values)

    def __repr__(self):
        return "lymph.config.Configuration(values={values})".format(
            values=self.values)



from __future__ import absolute_import

import logging
import pdb
import six
import socket
import sys


logger = logging.getLogger(__name__)


class Ripdb(pdb.Pdb):
    """
    Based on
        * https://github.com/tamentis/rpdb
        * http://blog.ionelmc.ro/2013/06/05/python-debugging-tools/

    """
    def __init__(self, port=0):
        self.old_stdout = sys.stdout
        self.old_stdin = sys.stdin
        self.listen_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.listen_socket.bind(('0.0.0.0', port))
        if not port:
            logger.critical("PDB remote session open on: %s", self.listen_socket.getsockname())
            six.print_("PDB remote session open on:", self.listen_socket.getsockname(), file=sys.__stderr__)
            sys.stderr.flush()
        self.listen_socket.listen(1)
        self.connected_socket, address = self.listen_socket.accept()
        self.handle = self.connected_socket.makefile('rw')
        pdb.Pdb.__init__(self, stdin=self.handle, stdout=self.handle)
        sys.stdout = sys.stdin = self.handle

    def do_continue(self, arg):
        sys.stdout = self.old_stdout
        sys.stdin = self.old_stdin
        self.handle.close()
        self.connected_socket.close()
        self.listen_socket.close()
        self.set_continue()
        return 1

    do_c = do_cont = do_continue


def set_trace():
    Ripdb().set_trace(sys._getframe().f_back)



from __future__ import absolute_import

from kombu.serialization import BytesIO

from lymph.serializers.base import msgpack_serializer, json_serializer


def _load_msgpack(s):
    return msgpack_serializer.load(BytesIO(s))


def _load_json(s):
    return json_serializer.load(BytesIO(s))


json_serializer_args = (json_serializer.dumps, _load_json, 'application/lymph+json', 'utf-8')
msgpack_serializer_args = (msgpack_serializer.dumps, _load_msgpack, 'application/lymph+x-msgpack', 'binary')



import hashlib
import logging

import six

from lymph.utils import observables


logger = logging.getLogger(__name__)

# Event types propagated by Service when instances change.
ADDED = 'ADDED'
REMOVED = 'REMOVED'
UPDATED = 'UPDATED'


class ServiceInstance(object):
    def __init__(self, endpoint=None, identity=None, **info):
        self.identity = identity if identity else hashlib.md5(endpoint.encode('utf-8')).hexdigest()
        self.info = info
        self.update(endpoint, **info)

    def update(self, endpoint, **info):
        self.endpoint = endpoint
        self.__dict__.update(info)
        self.info.update(info)

    def serialize(self):
        d = {
            'endpoint': self.endpoint,
            'identity': self.identity,
            'log_endpoint': self.log_endpoint,
            'fqdn': self.fqdn,
        }
        d.update(self.info)
        return d


class Service(observables.Observable):
    def __init__(self, name=None, instances=()):
        super(Service, self).__init__()
        self.name = name
        self.instances = {i.endpoint: i for i in instances}

    def __iter__(self):
        return six.itervalues(self.instances)

    def __len__(self):
        return len(self.instances)

    def get_instance(self, identity_prefix):
        for instance in six.itervalues(self.instances):
            if instance.identity.startswith(identity_prefix):
                return instance

    def identities(self):
        return list(self.instances.keys())

    def remove(self, identity):
        try:
            instance = self.instances.pop(identity)
        except KeyError:
            pass
        else:
            self.notify_observers(REMOVED, instance)

    def update(self, identity, **info):
        if identity in self.instances:
            self.instances[identity].update(**info)
            self.notify_observers(UPDATED, self.instances[identity])
        else:
            instance = self.instances[identity] = ServiceInstance(**info)
            self.notify_observers(ADDED, instance)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

import sys


with open('README.rst') as f:
    description = f.read()

install_requires = [
    'docopt>=0.6.1',
    'kazoo>=1.3.1',
    'kombu>=3.0.16',
    'gevent',
    'msgpack-python>=0.4.0',
    'psutil>=2.1.1',
    'PyYAML>=3.11',
    'pyzmq>=14.3.0',
    'redis>=2.9.1',
    'setproctitle>=1.1.8',
    'six>=1.6',
    'Werkzeug>=0.9.4',
    'blessings>=1.5.1',
    'netifaces>=0.10.4',
    'mock>=1.0.1',
    'PyHamcrest==1.8.2',
]

dependency_links = []

if sys.version_info.major == 2:
    install_requires.append('Monotime>=1.0')
elif sys.version_info.major == 3:
    # Installing Cython==0.20.1 for building gevent
    from setuptools.command.easy_install import main as easy_install
    easy_install(['Cython==0.20.1'])
    dependency_links.append(
        'git+https://github.com/gevent/gevent.git#egg=gevent-1.0.1')

setup(
    name='lymph',
    url='http://github.com/deliveryhero/lymph/',
    version='0.4.0',
    namespace_packages=['lymph'],
    packages=find_packages(),
    license=u'Apache License (2.0)',
    author=u'Delivery Hero Holding GmbH',
    maintainer=u'Johannes Dollinger',
    maintainer_email=u'johannes.dollinger@deliveryhero.com',
    long_description=description,
    include_package_data=True,
    install_requires=install_requires,
    dependency_links=dependency_links,
    extras_require={
        'sentry': ['raven'],
        'newrelic': ['newrelic'],
    },
    entry_points={
        'console_scripts': ['lymph = lymph.cli.main:main'],
        'lymph.cli': [
            'discover = lymph.cli.discover:DiscoverCommand',
            'emit = lymph.cli.emit:EmitCommand',
            'help = lymph.cli.help:HelpCommand',
            'inspect = lymph.cli.inspect:InspectCommand',
            'instance = lymph.cli.service:InstanceCommand',
            'list = lymph.cli.list:ListCommand',
            'node = lymph.cli.service:NodeCommand',
            'request = lymph.cli.request:RequestCommand',
            'shell = lymph.cli.shell:ShellCommand',
            'subscribe = lymph.cli.subscribe:SubscribeCommand',
            'tail = lymph.cli.tail:TailCommand',
        ],
        'nose.plugins.0.10': ['lymph = lymph.testing.nose:LymphPlugin'],
        'pytest11': ['lymph = lymph.testing.pytest'],
        'kombu.serializers': [
            'lymph-json = lymph.serializers.kombu:json_serializer_args',
            'lymph-msgpack = lymph.serializers.kombu:msgpack_serializer_args',
        ],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3'
    ]
)



from gevent.pool import Pool, Group

from lymph.exceptions import ResourceExhausted


class RejectExcecutionError(ResourceExhausted):
    pass


class NonBlockingPool(Pool):
    """A gevent pool that when exhausted will wait for a given timeout
    for resources to be freed before rejecting the job by raising
    exc:``RejectedExcecutionError``.

    When the ``timeout`` is not given or set to None the pool will reject
    immediately without waiting when pool size reach max size.

    In case ``size`` is None this will create an unbound pool.

    """

    def __init__(self, timeout=None, **kwargs):
        super(NonBlockingPool, self).__init__(**kwargs)
        self._timeout = timeout

    def add(self, greenlet):
        acquired = self._semaphore.acquire(blocking=False, timeout=self._timeout)
        # XXX(Mouad): Checking directly for False because DummySemaphore always
        # return None https://github.com/gevent/gevent/pull/544.
        if acquired is False:
            raise RejectExcecutionError('No more resource available to run %r' % greenlet)
        try:
            Group.add(self, greenlet)
        except:
            self._semaphore.release()
            raise



import abc
import collections
import functools
import inspect

import six

from lymph.core.declarations import Declaration


@six.add_metaclass(abc.ABCMeta)
class RPCBase(collections.Callable):
    """Base interface for RPC functions.

    Implementation of this interface can be used as decorator for
    functions/methods.

    Example ::

        >>> @Decorator
        ... def fibo(n):
        ...     "Fibonacci number."
        ...     if n <= 1: return n
        ...     return fibo(n-1) + fibo(n-2)
        ...
        >>> fibo.__doc__
        'Fibonacci number.'
        >>> fibo.__name__
        'fibo'
        >>> fibo(3)
        2
    """

    def __init__(self, func, assigned=functools.WRAPPER_ASSIGNMENTS):
        self.original = func
        self._func = func

        functools.update_wrapper(
            self, func, assigned=functools.WRAPPER_ASSIGNMENTS)

    @property
    def args(self):
        """Return original function argument spec skipping self.

        Returns:
          ``inspect.ArgSpec``.
        """
        spec = inspect.getargspec(self._func)
        return inspect.ArgSpec(spec.args[1:], *spec[1:])

    def __get__(self, obj, obj_type=None):
        if obj is None:
            return self
        return functools.partial(self._func, obj)

    def __call__(self, *args, **kwargs):
        return self._func(*args, **kwargs)

    @abc.abstractmethod
    def rpc_call(self, interface, channel, *args, **kwargs):
        pass

    def decorate(self, decorator):
        self._func = decorator(self._func)


class _RawRPCDecorator(RPCBase):

    @property
    def args(self):
        # Skip channel in the arguments spec.
        spec = super(_RawRPCDecorator, self).args
        return inspect.ArgSpec(spec.args[1:], *spec[1:])

    def rpc_call(self, interface, channel, *args, **kwargs):
        return self._func(interface, channel, *args, **kwargs)


class _RPCDecorator(RPCBase):

    def __init__(self, *args, **kwargs):
        self._raises = kwargs.pop('raises', ())
        super(_RPCDecorator, self).__init__(*args, **kwargs)

    @property
    def raises(self):
        return self._raises

    def rpc_call(self, interface, channel, *args, **kwargs):
        try:
            ret = self._func(interface, *args, **kwargs)
        except self._raises as ex:
            channel.error(type=ex.__class__.__name__, message=str(ex))
        else:
            channel.reply(ret)


def raw_rpc():
    return _RawRPCDecorator


def rpc(raises=()):
    return functools.partial(_RPCDecorator, raises=raises)


def event(*event_types, **kwargs):
    def decorator(func):
        from lymph.core.events import EventHandler
        if isinstance(func, EventHandler):
            raise TypeError('lymph.event() decorators cannot be stacked')

        def factory(interface):
            return EventHandler(interface, func, event_types, **kwargs)
        return Declaration(factory)
    return decorator




import unittest
import functools

from lymph.exceptions import RemoteError
from lymph.core.interfaces import Proxy
from lymph.core.container import ServiceContainer

import mock

import hamcrest


class MockMixins(object):

    def _assert_equal_calls(self, actual_calls, expected_calls):
        actual_count, expected_count = len(actual_calls), len(expected_calls)

        # TODO(Mouad): Add in the message which calls are missing.
        self.assertEqual(
            expected_count, actual_count,
            msg="number of calls doesn't match, expected %d actual %d" % (expected_count, actual_count))

        for pos, (actual_call, expected_call) in enumerate(zip(actual_calls, expected_calls)):
            success, msg = self._check_equal_call(actual_call, expected_call)
            if not success:
                self.fail("function #%d %s" % (pos, msg))

    def _check_equal_call(self, actual_call, expected_call):
        act_name, act_args, act_kwargs = self._unpack_mock_call(actual_call)
        exp_name, exp_args, exp_kwargs = self._unpack_mock_call(expected_call)

        if exp_name != act_name:
            return False, "name doesn't match, expected %r actual %r" % (exp_name, act_name)

        success, msg = self._check_equal_arguments(act_args, exp_args)
        if not success:
            return False, msg
        return self._check_equal_keyword_arguments(act_kwargs, exp_kwargs)

    @staticmethod
    def _unpack_mock_call(mock_call):
        # Since we are using the syntax mock.call('<func_name>', arg1, arg2, kw1=val1),
        # in this case the unpacking of mock.call will put the <func_name> in args[0] and not
        # in the first unpacked element.
        _, args, kwargs = mock_call
        name, args = args[0], args[1:]
        return name, args, kwargs

    def _check_equal_arguments(self, actual_args, expected_args):
        if len(actual_args) != len(expected_args):
            return False, "arguments count doesn't match, expected %r actual %r" % (len(expected_args), len(actual_args))

        for pos, (actual_arg, expected_arg) in enumerate(zip(actual_args, expected_args)):
            success, msg = self._check_that(actual_arg, hamcrest.is_(expected_arg))
            if not success:
                return False, "argument #%d doesn't match, %s" % (pos, msg)
        return True, None

    def _check_equal_keyword_arguments(self, actual_kwargs, expected_kwargs):
        actual_arg_names = sorted(actual_kwargs.keys())
        expected_arg_names = sorted(expected_kwargs.keys())

        if actual_arg_names != expected_arg_names:
            return False, "keyword arguments doesn't match, expected %r actual %r" % (expected_arg_names, actual_arg_names)

        for name, act_value in actual_kwargs.items():
            success, msg = self._check_that(act_value, hamcrest.is_(expected_kwargs[name]))
            if not success:
                return False, "keyword argument %r doesn't match, %s" % (name, msg)
        return True, None

    @staticmethod
    def _check_that(actual, expected):
        try:
            hamcrest.assert_that(actual, expected)
        except AssertionError as ex:
            msg = ex.args[0]
            return False, msg
        return True, None

    def _assert_equal_any_calls(self, actual_calls, expected_calls):
        next_idx = 0
        for exp_call in expected_calls:
            name, _, _ = self._unpack_mock_call(exp_call)
            possible_matches = self._get_same_named_calls(actual_calls, name, start_idx=next_idx)
            if not possible_matches:
                self.fail("Call '%r' wasn't found." % (exp_call, ))
            elif len(possible_matches) == 1:
                # In case we have one possible match, use _assert_equal_calls to have
                # a better error message.
                match_idx, actual_call = possible_matches[0]
                self._assert_equal_calls([actual_call], [exp_call])
            else:
                match_idx = self._assert_match_one_call(possible_matches, exp_call)
            next_idx = match_idx + 1

    def _get_same_named_calls(self, actual_calls, name, start_idx=0):
        matches = []
        for i, act_call in enumerate(actual_calls[start_idx:]):
            act_name, _, _ = self._unpack_mock_call(act_call)
            if name == act_name:
                matches.append((i + start_idx, act_call))
        return matches

    def _assert_match_one_call(self, actual_calls, exp_call):
        for idx, act_call in actual_calls:
            success, _ = self._check_equal_call(act_call, exp_call)
            if success:
                return idx
        msg = "Call '%r' wasn't found." % (exp_call, )
        if actual_calls:
            msg += " Maybe you want: \n%s" % "\n".join(map(str, actual_calls))
        self.fail(msg)


def _get_rpc_mock(rpc_mocks={}, original=Proxy._call):
    class ProxyCall(mock.MagicMock):

        rpc_functions = rpc_mocks

        def __call__(self, proxy_inst, __name, **kwargs):
            # XXX (Mouad): We need to call MagicMock __call__ here else calls
            # will not be tracked, and we do it for all calls mocked or not.
            super(ProxyCall, self).__call__(__name, **kwargs)
            try:
                result = self.rpc_functions[__name]
            except KeyError:
                return original(proxy_inst, __name, **kwargs)
            else:
                if isinstance(result, Exception):
                    raise getattr(RemoteError, result.__class__.__name__)('', '')
                if callable(result):
                    return result(**kwargs)
                return result

        def __get__(self, obj, type=None):
            return functools.partial(self.__call__, obj)
    return ProxyCall()


class RpcMockTestCase(unittest.TestCase, MockMixins):
    def setUp(self):
        super(RpcMockTestCase, self).setUp()
        self.rpc_patch = mock.patch.object(Proxy, '_call', new_callable=_get_rpc_mock)
        self.rpc_mock = self.rpc_patch.start()

    def tearDown(self):
        super(RpcMockTestCase, self).tearDown()
        self.rpc_patch.stop()

    def setup_rpc_mocks(self, rpc_functions):
        self.rpc_mock.rpc_functions = rpc_functions

    def update_rpc_mock(self, func_name, return_value):
        self.rpc_mock.rpc_functions[func_name] = return_value

    def delete_rpc_mock(self, func_name):
        del self.rpc_mock.rpc_functions[func_name]

    @property
    def rpc_mock_calls(self):
        return self.rpc_mock.mock_calls

    def assert_rpc_calls(self, *expected_calls):
        self._assert_equal_calls(self.rpc_mock_calls, expected_calls)

    def assert_any_rpc_calls(self, *expected_calls):
        self._assert_equal_any_calls(self.rpc_mock_calls, expected_calls)


class EventMockTestCase(unittest.TestCase, MockMixins):
    def setUp(self):
        super(EventMockTestCase, self).setUp()
        self.event_patch = mock.patch.object(ServiceContainer, 'emit_event')
        self.event_mock = self.event_patch.start()

    def tearDown(self):
        super(EventMockTestCase, self).tearDown()
        self.event_patch.stop()

    @property
    def events(self):
        return self.event_mock.mock_calls

    def assert_events_emitted(self, *expected_emitted):
        expected_emitted_with_delay = []
        for call in expected_emitted:
            name, args, kwargs = self._unpack_mock_call(call)
            kwargs.setdefault('delay', 0)
            expected_emitted_with_delay.append(mock.call(name, *args, **kwargs))
        self._assert_equal_calls(self.events, expected_emitted_with_delay)



from lymph.core.components import Declaration


def proxy(*args, **kwargs):
    def factory(interface):
        from lymph.core.interfaces import Proxy
        return Proxy(interface.container, *args, **kwargs)
    return Declaration(factory)



import gevent
import uuid
import kombu

import lymph

from lymph.events.kombu import KombuEventSystem
from lymph.discovery.static import StaticServiceRegistryHub
from lymph.testing import LymphIntegrationTestCase, AsyncTestsMixin


class TestInterface(lymph.Interface):
    def __init__(self, *args, **kwargs):
        super(TestInterface, self).__init__(*args, **kwargs)
        self.collected_events = []

    @lymph.event('foo')
    def on_foo(self, event):
        self.collected_events.append(event)

    @lymph.event('foo_broadcast', broadcast=True)
    def on_foo_broadcast(self, event):
        self.collected_events.append(event)


class TestEventBroadcastInterface(lymph.Interface):
    def __init__(self, *args, **kwargs):
        super(TestEventBroadcastInterface, self).__init__(*args, **kwargs)
        self.collected_events = []

    @lymph.event('foo_broadcast', broadcast=True)
    def on_foo_broadcast(self, event):
        self.collected_events.append(event)


class KombuIntegrationTest(LymphIntegrationTestCase, AsyncTestsMixin):
    use_zookeeper = False

    def setUp(self):
        super(KombuIntegrationTest, self).setUp()
        self.exchange_name = 'test-%s' % uuid.uuid4()
        self.discovery_hub = StaticServiceRegistryHub()

        self.the_container, self.the_interface = self.create_container(TestInterface, 'test')
        self.the_container_broadcast, self.the_interface_broadcast = self.create_container(TestEventBroadcastInterface, 'test')
        self.lymph_client = self.create_client()

    def tearDown(self):
        super(KombuIntegrationTest, self).tearDown()
        connection = self.get_kombu_connection()
        exchange = kombu.Exchange(self.exchange_name)
        exchange(connection).delete()

        # FIXME: there should be a better way to get this exchange name
        waiting_exchange = kombu.Exchange('%s_waiting' % self.exchange_name)
        waiting_exchange(connection).delete()

        # FIXME: there should be a better way to get the queue names
        for name in ('foo-wait_500', 'test-on_foo'):
            queue = kombu.Queue(name)
            queue(connection).delete()

    def get_kombu_connection(self):
        return kombu.Connection(transport='amqp', host='127.0.0.1')

    def create_event_system(self, **kwargs):
        return KombuEventSystem(self.get_kombu_connection(), self.exchange_name)

    def create_registry(self, **kwargs):
        return self.discovery_hub.create_registry(**kwargs)

    def received_check(self, n):
        def check():
            return len(self.the_interface.collected_events) == n
        return check

    def received_broadcast_check(self, n):
        def check():
            return (len(self.the_interface.collected_events) + len(self.the_interface_broadcast.collected_events)) == n
        return check

    def test_emit(self):
        self.lymph_client.emit('foo', {})
        self.assert_eventually_true(self.received_check(1), timeout=10)
        self.assertEqual(self.the_interface.collected_events[0].evt_type, 'foo')

    def test_delayed_emit(self):
        self.lymph_client.emit('foo', {}, delay=.5)
        self.assert_temporarily_true(self.received_check(0), timeout=.2)
        self.assert_eventually_true(self.received_check(1), timeout=10)
        self.assertEqual(self.the_interface.collected_events[0].evt_type, 'foo')

    def test_broadcast_event(self):
        self.lymph_client.emit('foo_broadcast', {})
        self.assert_eventually_true(self.received_broadcast_check(2), timeout=10)
        self.assertEqual(self.the_interface.collected_events[0].evt_type, 'foo_broadcast')
        self.assertEqual(self.the_interface_broadcast.collected_events[0].evt_type, 'foo_broadcast')



import hashlib
import logging
import collections

import lymph
import gevent
from kazoo.handlers.gevent import SequentialGeventHandler

from lymph.core.declarations import Declaration
from lymph.core.events import Event
from lymph.core.interfaces import Component


logger = logging.getLogger(__name__)


def serial_event(*event_types, **kwargs):
    if 'key' not in kwargs:
        raise TypeError('key argument is required')

    def decorator(func):
        def factory(interface):
            zkclient = interface.config.root.get_instance(
                'components.SerialEventHandler.zkclient',
                handler=SequentialGeventHandler())
            return SerialEventHandler(zkclient, interface, func, event_types, **kwargs)
        return Declaration(factory)
    return decorator


class SerialEventHandler(Component):
    def __init__(self, zkclient, interface, func, event_types, key, partition_count=12):
        super(SerialEventHandler, self).__init__()
        self.zk = zkclient
        self.interface = interface
        self.partition_count = partition_count
        self.key = key
        self.consumer_func = func
        self.consumers = collections.OrderedDict()
        self.name = '%s.%s' % (interface.name, func.__name__)

        def _consume(interface, event):
            self.consumer_func(self.interface, Event.deserialize(event['event']))

        for i in range(partition_count):
            queue = self.get_queue_name(i)
            e = lymph.event(queue, queue_name=queue, sequential=True, active=False)(_consume)
            handler = e.install(interface)
            self.consumers[handler] = interface.container.subscribe(handler, consume=False)
        self.partition = set()
        push_queue = self.get_queue_name('push')
        lymph.event(*event_types, queue_name=push_queue)(self.push).install(interface)

    def on_start(self):
        super(SerialEventHandler, self).on_start()
        self.start()

    def on_stop(self, **kwargs):
        super(SerialEventHandler, self).on_stop(**kwargs)
        self.running = False

    def get_queue_name(self, index):
        return '%s.%s' % (self.consumer_func.__name__, index)

    def push(self, interface, event):
        key = str(self.key(interface, event)).encode('utf-8')
        index = int(hashlib.md5(key).hexdigest(), 16) % self.partition_count
        logger.debug('PUBLISH %s %s', self.get_queue_name(index), event)
        self.interface.emit(self.get_queue_name(index), {'event': event.serialize()})

    def start(self):
        self.running = True
        self.interface.container.spawn(self.loop)

    def loop(self):
        while self.running:
            logger.info('starting partitioner')
            partitioner = self.zk.SetPartitioner(
                path='/lymph/serial_event_partitions/%s' % self.name,
                set=self.consumers.keys(),
                time_boundary=1,
            )
            while True:
                if partitioner.failed:
                    logger.error('partitioning failed')
                    break
                elif partitioner.release:
                    self.release_partition()
                    partitioner.release_set()
                elif partitioner.acquired:
                    self.update_partition(set(partitioner))
                    gevent.sleep(1)
                elif partitioner.allocating:
                    partitioner.wait_for_acquire()

    def release_partition(self):
        for handler in self.partition:
            self.stop_consuming(handler)
        self.partition = set()

    def update_partition(self, partition):
        for queue in self.partition - partition:
            self.stop_consuming(queue)
        for queue in partition - self.partition:
            self.start_consuming(queue)
        if partition != self.partition:
            logger.info('partition: %s', ', '.join(h.queue_name for h in partition))
        self.partition = partition

    def start_consuming(self, handler):
        self.consumers[handler].start()

    def stop_consuming(self, handler):
        self.consumers[handler].stop()





def patch():
    if patch._initialized:
        return
    patch._initialized = True

    import gevent.monkey
    gevent.monkey.patch_all()

    import sys
    if sys.version_info.major < 3:
        _py2_patches()

    _export()
patch._initialized = False


def _export():
    import lymph
    lymph.__version__ = '0.4.0'

    from lymph.exceptions import RpcError, LookupFailure, Timeout
    from lymph.core.decorators import rpc, raw_rpc, event
    from lymph.core.interfaces import Interface
    from lymph.core.declarations import proxy

    for obj in (RpcError, LookupFailure, Timeout, rpc, raw_rpc, event, Interface, proxy):
        setattr(lymph, obj.__name__, obj)


def _py2_patches():
    import monotime  # NOQA



from werkzeug.routing import Rule


class HandledRule(Rule):

    def __init__(self, string, handler, **kwargs):
        self.handler = handler
        super(HandledRule, self).__init__(string, **kwargs)

    def get_empty_kwargs(self):
        empty_kwargs = super(HandledRule, self).get_empty_kwargs()
        empty_kwargs['handler'] = self.handler
        return empty_kwargs




import re
import logging
from uuid import uuid4
from lymph.core.interfaces import Component
from lymph.core import trace


logger = logging.getLogger(__name__)


class Event(object):
    def __init__(self, evt_type, body, source=None, headers=None, event_id=None):
        self.event_id = event_id
        self.evt_type = evt_type
        self.body = body
        self.source = source
        self.headers = headers or {}

    def __getitem__(self, key):
        return self.body[key]

    def __iter__(self):
        return iter(self.body)

    def __repr__(self):
        return '<Event type=%r body=%r>' % (self.evt_type, self.body)

    def __str__(self):
        return '{type=%s id=%s}' % (self.evt_type, self.event_id)

    @classmethod
    def deserialize(cls, data):
        return cls(data.get('type'), data.get('body', {}), source=data.get('source'), headers=data.get('headers'))

    def serialize(self):
        return {
            'type': self.evt_type,
            'headers': self.headers,
            'body': self.body,
            'source': self.source,
        }


class EventHandler(Component):
    def __init__(self, interface, func, event_types, sequential=False, queue_name=None, active=True, once=False, broadcast=False):
        assert not (once and broadcast), "Once and broadcast cannot be enabled at the same time"
        super(EventHandler, self).__init__()
        self.func = func
        self.event_types = event_types
        self.sequential = sequential
        self.active = active
        self.interface = interface
        self.once = once
        self.broadcast = broadcast
        self.unique_key = str(uuid4()) if once or broadcast else None
        self._queue_name = queue_name or func.__name__

    @property
    def queue_name(self):
        if self.unique_key:
            return '%s-%s-%s' % (self.interface.name, self._queue_name, self.unique_key)
        else:
            return '%s-%s' % (self.interface.name, self._queue_name)

    @queue_name.setter
    def queue_name(self, value):
        self._queue_name = value

    def on_start(self):
        self.interface.container.subscribe(self, consume=self.active)

    def __call__(self, event, *args, **kwargs):
        trace.set_id(event.headers.get('trace_id'))
        logger.debug('<E %s', event)
        return self.func(self.interface, event, *args, **kwargs)


class EventDispatcher(object):
    wildcards = {
        '#': r'[\w.]*(?=\.|$)',
        '*': r'\w+',
    }

    def __init__(self, patterns=()):
        self.patterns = []
        self.update(patterns)

    def compile(self, key):
        words = (self.wildcards.get(word, re.escape(word)) for word in key.split('.'))
        return re.compile('^%s$' % r'\.'.join(words))

    def register(self, pattern, handler):
        self.patterns.append((
            self.compile(pattern),
            pattern,
            handler,
        ))

    def __iter__(self):
        for regex, pattern, handler in self.patterns:
            yield pattern, handler

    def update(self, other):
        for pattern, handler in other:
            self.register(pattern, handler)

    def dispatch(self, evt_type):
        for regex, pattern, handler in self.patterns:
            if regex.match(evt_type):
                yield pattern, handler

    def __call__(self, event):
        handlers = set()
        for pattern, handler in self.dispatch(event.evt_type):
            if handler not in handlers:
                handlers.add(handler)
                handler(event)
        return bool(handlers)



import collections
from contextlib import contextmanager
import sys
import os
import tempfile
import textwrap

from kazoo.client import KazooClient
from kazoo.handlers.gevent import SequentialGeventHandler
from pkg_resources import load_entry_point
from six import StringIO, integer_types
import yaml

from lymph.discovery.zookeeper import ZookeeperServiceRegistry
from lymph.events.null import NullEventSystem
from lymph.testing import LymphIntegrationTestCase


@contextmanager
def capture_output():
    real_stdout = sys.stdout
    real_stderr = sys.stderr
    sys.stdout = StringIO()
    sys.stderr = StringIO()
    try:
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout = real_stdout
        sys.stderr = real_stderr


class CliWrapper(object):

    Result = collections.namedtuple('CMDLineResult', 'returncode stdout stderr')

    def __init__(self, config):
        handle, self.config_file_name = tempfile.mkstemp()
        with open(self.config_file_name, 'w') as f:
            f.write(yaml.dump(config))
        self.entry_point = load_entry_point('lymph', 'console_scripts', 'lymph')

    def tear_down(self):
        os.remove(self.config_file_name)

    def __call__(self, cmd, config=True):
        with capture_output() as (stdout, stderr):
            if config:
                cmd = cmd + ['--config=%s' % self.config_file_name]
            try:
                returncode = self.entry_point(cmd)
            except SystemExit as ex:
                # Docopt tries to exit on its own unfortunately
                returncode = (ex.args[0] or 0) if ex.args else 0
                if not isinstance(returncode, integer_types):
                    # According to sys.exit doc, any other object beside
                    # an integer or None result to an exit code equal to 1.
                    returncode = 1
            return self.Result(
                returncode or 0, stdout.getvalue(), stderr.getvalue())


class CliTestMixin(object):

    cli_config = {}

    _help_output = None

    def setUp(self):
        self.__clis = []
        super(CliTestMixin, self).setUp()

    @property
    def cli(self):
        cli = CliWrapper(self.cli_config)
        self.__clis.append(cli)
        return cli

    def tearDown(self):
        for cli in self.__clis:
            cli.tear_down()
        super(CliTestMixin, self).tearDown()

    def assert_lines_equal(self, cmd, lines, config=True):
        expected_lines = set(line for line in textwrap.dedent(lines).splitlines() if line.strip())
        result = self.cli(cmd, config=config)
        self.assertEqual(result.returncode, 0)
        self.assertEqual(set(result.stdout.splitlines()), expected_lines)

    def assert_first_line_equals(self, cmd, line, config=True):
        result = self.cli(cmd, config=config)
        self.assertEqual(result.returncode, 0)
        self.assertEqual(result.stdout.splitlines()[0].strip(), line)

    def assert_command_appears_in_command_list(self):
        result = self.cli(['list'])
        self.assertEqual(result.returncode, 0)
        self.assertIn(self.command_name, result.stdout)

    def assert_help_contains_usage_information(self):
        output = self._get_help_output()
        self.assertIn('Usage', output)
        self.assertIn(self.command_name, output)

    def assert_help_contains_parameter(self, parameter, default=None):
        self.assert_help_contains(parameter)
        if default is not None:
            self.assert_help_contains(default)

    def assert_help_contains(self, text):
        self.assertIn(text, self._get_help_output())

    def _get_help_output(self):
        if self._help_output is None:
            result = self.cli([self.command_name, '--help'])
            self._help_output = result.stdout
        return self._help_output


class CliIntegrationTestCase(CliTestMixin, LymphIntegrationTestCase):
    use_zookeeper = True

    def setUp(self):
        super(CliIntegrationTestCase, self).setUp()
        client = KazooClient(
            hosts=self.hosts,
            handler=SequentialGeventHandler(),
        )
        self.registry = ZookeeperServiceRegistry(client)
        self.events = NullEventSystem()

        self.cli_config = {
            "container": {
                "registry": {
                    "class": "lymph.discovery.zookeeper:ZookeeperServiceRegistry",
                    "zkclient": 'dep:kazoo',
                },
            },
            "event_system": {
                "class": "lymph.events.null:NullEventSystem",
            },
            "dependencies": {
                "kazoo": {
                    "class": "kazoo.client:KazooClient",
                    "hosts": self.hosts,
                }
            }
        }


class CommandFactory(object):
    """
    Encapsulates the knowledge how to create a command instance.

    Intended use is to support smaller unit tests which just need an instance
    of a command class to try out some method.

    It only supports to pass in parameters as keyword parameters into
    the command constructor.
    """

    def __init__(self, command_class):
        self.command_class = command_class

    def __call__(self, **kwargs):
        kwargs.setdefault('args', {})
        kwargs.setdefault('config', {})
        kwargs.setdefault('terminal', None)
        return self.command_class(**kwargs)



from unittest import TestCase

from lymph.utils.sockets import guess_external_ip


class SocketUtilsTests(TestCase):
    def test_guess_external_ip(self):
        self.assertNotEqual(guess_external_ip(), '127.0.0.1')



from __future__ import absolute_import

import logging

log = logging.getLogger(__name__)


def pytest_configure(config):
    log.info("Initializing the lymph framework")
    import lymph.monkey
    lymph.monkey.patch()




import logging
from unittest import TestCase

from lymph.utils.logging import get_loglevel


class LoggingUtilsTests(TestCase):
    def test_get_loglevel(self):
        self.assertEqual(get_loglevel('DEBUG'), logging.DEBUG)
        self.assertEqual(get_loglevel('debug'), logging.DEBUG)
        self.assertEqual(get_loglevel('Debug'), logging.DEBUG)
        self.assertEqual(get_loglevel('INFO'), logging.INFO)
        self.assertEqual(get_loglevel('info'), logging.INFO)
        self.assertEqual(get_loglevel('ERROR'), logging.ERROR)
        self.assertEqual(get_loglevel('error'), logging.ERROR)
        self.assertEqual(get_loglevel('CRITICAL'), logging.CRITICAL)
        self.assertEqual(get_loglevel('critical'), logging.CRITICAL)

        self.assertRaises(ValueError, get_loglevel, 'FOO')
        self.assertRaises(ValueError, get_loglevel, '*')



import unittest
import textwrap
import yaml

from lymph.config import Configuration
from lymph.exceptions import ConfigurationError


class ConfigurationTests(unittest.TestCase):
    def load_yaml(self, yml):
        return Configuration(yaml.load(textwrap.dedent(yml)))

    def test_setdefault_without_previous_value(self):
        config = Configuration({
            'nested': {}
        })
        config.setdefault('foo', 1)
        self.assertEqual(config.get('foo'), 1)
        config.setdefault('nested.foo', 2)
        self.assertEqual(config.get('nested.foo'), 2)

    def test_setdefault_with_previous_value(self):
        config = Configuration({
            'foo': 1,
            'nested': {
                'foo': 2
            }
        })
        config.setdefault('foo', 0)
        self.assertEqual(config.get('foo'), 1)
        config.setdefault('nested.foo', 0)
        self.assertEqual(config.get('nested.foo'), 2)

    def test_get_missing_key(self):
        config = Configuration({'a': {'b': 1}, 'c': None})

        self.assertIsNone(config.get('x'))
        self.assertEqual(config.get('x', 2), 2)

        self.assertIsNone(config.get('x.x'), None)
        self.assertEqual(config.get('x.x', 2), 2)

        self.assertIsNone(config.get('c.x'), None)
        self.assertEqual(config.get('c.x', 2), 2)

    def test_set_in_empty_sections(self):
        config = self.load_yaml("""
        foo:
        """)
        # yaml doesn't know that we expect `foo` to be a dict
        self.assertIsNone(config.get('foo'))
        config.set('foo.bar', 1)
        self.assertEqual(config.get('foo.bar'), 1)

    def test_update(self):
        config = Configuration({'a': 1, 'b': {'c': 2}})
        config.update({'a': 2})
        self.assertEqual(config.get('a'), 2)
        self.assertEqual(config.get('b.c'), 2)

    def test_contains(self):
        config = Configuration({'a': {'b': 1}})
        self.assertTrue('a' in config)
        self.assertFalse('foo' in config)
        view = config.get('a')
        self.assertTrue('b' in view)
        self.assertFalse('foo' in view)

    def test_env_replacement(self):
        config = Configuration({
            'replace': 'prefix_$(env.FOO_BAR)_suffix',
            'type_conversion': 'prefix_$(env.FOURTYTWO)',
            'keep_int': '$(env.FOURTYTWO)',
            'keep_dict': '$(env.DICT)',
            'no_parens': '$env.FOO_BAR',
            'no_namespace': '$(FOO_BAR)',
            'nested': '$(var.nested.foo)',
        }, env={
            'FOO_BAR': '42',
            'FOURTYTWO': 42,
            'DICT': {},
        }, var=Configuration({
            'nested': {
                'foo': 1764,
            }
        }))
        self.assertEqual(config.get('replace'), 'prefix_42_suffix')
        self.assertEqual(config.get('type_conversion'), 'prefix_42')
        self.assertEqual(config.get('keep_int'), 42)
        self.assertEqual(config.get('keep_dict'), {})
        self.assertEqual(config.get('no_parens'), '$env.FOO_BAR')
        self.assertEqual(config.get('no_namespace'), '$(FOO_BAR)')
        self.assertEqual(config.get('nested'), 1764)

    def test_missing_env_replacement(self):
        self.assertRaises(ConfigurationError, Configuration, {
            'missing': '$(env.MISSING)'
        }, env={})
        self.assertRaises(ConfigurationError, Configuration, {
            'bad_namespace': '$(env.FOO_BAR)',
        }, other={})


class ConfigurableThing(object):
    def __init__(self, config):
        self.config = config

    @classmethod
    def from_config(cls, config):
        return cls(dict(config.items()))


class CreateInstanceTest(unittest.TestCase):
    config = Configuration({
        "thing": {
            "class": "%s:ConfigurableThing" % __name__,
            "param": 41
        },
        "section": {
            "thing": {
                "class": "%s:ConfigurableThing" % __name__,
                "param": 42
            },
        }
    })

    def test_create_instance(self):
        obj = self.config.create_instance('thing')
        self.assertIsInstance(obj, ConfigurableThing)
        self.assertEqual(obj.config['param'], 41)

    def test_create_instance_on_view(self):
        config = self.config.get('section')
        obj = config.create_instance('thing')
        self.assertIsInstance(obj, ConfigurableThing)
        self.assertEqual(obj.config['param'], 42)


class GetInstanceTest(unittest.TestCase):
    config = Configuration({
        "thing": {
            "class": "%s:ConfigurableThing" % __name__,
        }
    })

    def test_creates_instance_based_on_configuration(self):
        instance = self.config.get_instance("thing")
        self.assertIsInstance(instance, ConfigurableThing)

    def test_returns_the_same_instance_on_multiple_invocations(self):
        instance_1 = self.config.get_instance("thing")
        instance_2 = self.config.get_instance("thing")
        self.assertIs(instance_1, instance_2)


class GetDependencyTest(unittest.TestCase):
    config = Configuration({
        "dependencies": {
            "thing": {
                "class": "%s:ConfigurableThing" % __name__
            },
        },
        "key": {
            "client": "dep:thing",
        }
    })

    def test_creates_instance_based_on_configuration(self):
        instance = self.config.get_instance("key.client")
        self.assertIsInstance(instance, ConfigurableThing)

    def test_returns_the_same_instance_on_multiple_invocations(self):
        instance_1 = self.config.get_instance("key.client")
        instance_2 = self.config.get_instance("key.client")
        self.assertIs(instance_1, instance_2)



from lymph.core.container import create_container
from lymph.core.interfaces import Interface


class ClientInterface(Interface):
    register_with_coordinator = False


class Client(object):
    def __init__(self, container, interface=ClientInterface):
        self.container = container
        self.interface = container.install_interface(interface, name='_client')

    @classmethod
    def from_config(cls, config, **kwargs):
        interface_cls = kwargs.pop('interface_cls', ClientInterface)
        container = create_container(config)
        client = cls(container, interface_cls)
        container.start(register=False)
        return client

    def __getattr__(self, name):
        # FIXME: explicit is better than implicit
        return getattr(self.interface, name)





import six

from lymph.cli.base import Command, get_command_classes


class ListCommand(Command):
    """
    Usage: lymph list [options]

    {COMMON_OPTIONS}
    """

    short_description = 'List available commands.'
    needs_config = False

    def run(self):
        command_names = get_command_classes().keys()
        max_command_name = max(command_names, key=len)
        description_offset = len(max_command_name) + 2
        for name, cls in six.iteritems(get_command_classes()):
            print(u'{t.bold}{name:<{offset}}{t.normal}{description}'.format(
                t=self.terminal,
                name=name,
                description=cls.short_description,
                offset=description_offset
            ))

